@@ -8,11 +8,12 @@
 #include "treeStruct/inferenceSamples.h"
 #include "treeStruct/padNodeStat.h"
 #include "treeStruct/treeBin2.h"
+#include <vector>
 
 class improv8: public padForest 
 {
 	private:
-		treeBin2** forestRoots = NULL;
+		std::vector<treeBin2> forestRoots;
 		int numOfBins;
 
 	public:
@@ -20,8 +21,9 @@ class improv8: public padForest
 		improv8(const std::string& forestFileName);
 		~improv8();
 		void makePredictions(const inferenceSamples& observations);
-void makePrediction(double* observation, double* preds, int numFeatures, int numObservations, int numCores);
+		void makePrediction(double* observation, double* preds, int numFeatures, int numObservations, int numCores);
 		int makePrediction(double*& observation);
+		int makePrediction(std::vector<double>& observation);
 		int makePrediction(double*& observation, int numCore);
 		void writeForest(const std::string& forestFileName);
 		void changeNumBins(int newBinNum){
